#include <AccelStepper.h>

#define DirectionPin  4
#define PulsePin  13
#define EnablePin 5
#define CWPin 6
#define CCWPin 7
#define HEsensorPin 10
#define LEDPin 9

#include <EEPROM.h>

// EEPROM addresses
#define FOCUSER_POS_START 0
#define STEPPER_SPEED_ADD 3      

AccelStepper stepper(1, PulsePin, DirectionPin);

// Global vars
boolean moving = false;
boolean hold = false;
boolean reversed;
boolean positionSaved;               // Flag indicates if stepper position was saved as new focuser position
boolean firstPrint = false;
boolean homeFound = true;
String inputString;                  // Serial input command string (terminated with \n)
boolean findingHome = false;

// *************  this value must be changed for each specific setup ***************************************
long maxSteps = 505960;  // **** steps needed for 2 complete revolutions.  prevent cord wrap.
long minStep = 10000;
//  Must also enter this value/720 = steps/degree for first time ASCOM driver setup, click "properties 
// ******************************************************************************************************

int manualMoveStepSize = 50;  //higher number for faster manual moves
int findHomeStepSize = 100; //adjusts finding home speed

int buttonStateCW = HIGH;
int buttonStateCCW = HIGH;
boolean manualMoveCW = false;
boolean manualMoveCCW = false;

int HEState = 0;
long lastDebounceTime = 0;
long debounceDelay = 20;
void loop(){
  HEState = digitalRead(HEsensorPin);  //goes low by magnet
 
  if (HEState == LOW) {     
    // turn LED on:    
    digitalWrite(LEDPin, HIGH);  
  } else {
    // turn LED off:
    digitalWrite(LEDPin, LOW); 
  }
  
  if (findingHome == true){
   findHome();
  } 
  
  // Check for CCW manual move 
  buttonStateCCW = digitalRead(CCWPin);
  if (millis() - lastDebounceTime > debounceDelay){ 
    if (buttonStateCCW == LOW && manualMoveCCW == false){
      manualMoveCCW = true;
      lastDebounceTime = millis();
    }
    if (buttonStateCCW == HIGH && manualMoveCCW == true){
      manualMoveCCW = false;
      lastDebounceTime = millis();
    }
  }  
  
  if (manualMoveCCW == true){
    stepper.move(-manualMoveStepSize); // change number here to change manual move speed
    positionSaved = false;
  }
  
  // Check for CCW manual move 
  buttonStateCW = digitalRead(CWPin);
  if (millis() - lastDebounceTime > debounceDelay){ 
    if (buttonStateCW == LOW && manualMoveCW == false){
      manualMoveCW = true;
     lastDebounceTime = millis();
    }
    if (buttonStateCW == HIGH && manualMoveCW == true){
      manualMoveCW = false;
      lastDebounceTime = millis();
    }
  } 
  
  if (manualMoveCW == true){
    stepper.move(manualMoveStepSize); // change number here to change manual move speed
    //manualMoveCW = true;
    //Serial.println("CW move");
    positionSaved = false;
  }

  // Stepper loop
  if (stepper.distanceToGo() == 0){
    if (hold == false){
      digitalWrite(EnablePin, HIGH);
    }
  } else{
    digitalWrite(EnablePin, LOW);
  }
  if (stepper.distanceToGo() == 0 && !positionSaved){
    saveFocuserPos(stepper.currentPosition());
    positionSaved = true;
  }
  if(stepper.distanceToGo() != 0 && !firstPrint) { 
    firstPrint = true;
  }

  if(stepper.distanceToGo() != 0){
    moving = true;
  }else{
    moving = false;
  }
  stepper.run();
}
